/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef __REF_H
#define __REF_H

#include "qcommon.h"

#define	MAX_DLIGHTS		32
#define	MAX_ENTITIES	128
#define	MAX_PARTICLES	4096
#define	MAX_LIGHTSTYLES	256

#define POWERSUIT_SCALE		4.0F

#define SHELL_RED_COLOR		0xF2
#define SHELL_GREEN_COLOR	0xD0
#define SHELL_BLUE_COLOR	0xF3

#define SHELL_RG_COLOR		0xDC
//#define SHELL_RB_COLOR		0x86
#define SHELL_RB_COLOR		0x68
#define SHELL_BG_COLOR		0x78

//ROGUE
#define SHELL_DOUBLE_COLOR	0xDF // 223
#define	SHELL_HALF_DAM_COLOR	0x90
#define SHELL_CYAN_COLOR	0x72
//ROGUE

#define SHELL_WHITE_COLOR	0xD7

typedef struct entity_s
{
	struct model_s		*model;			// opaque type outside refresh
	float				angles[3];

	/*
	** most recent data
	*/
	float				origin[3];		// also used as RF_BEAM's "from"
	int					frame;			// also used as RF_BEAM's diameter

	/*
	** previous data for lerping
	*/
	float				oldorigin[3];	// also used as RF_BEAM's "to"
	int					oldframe;

	/*
	** misc
	*/
	float	backlerp;				// 0.0 = current, 1.0 = old
	int		skinnum;				// also used as RF_BEAM's palette index

	int		lightstyle;				// for flashing entities
	float	alpha;					// ignore if RF_TRANSLUCENT isn't set

	struct image_s	*skin;			// NULL for inline skin
	int		flags;

} entity_t;

#define ENTITY_FLAGS  68

typedef struct
{
	vec3_t	origin;
	vec3_t	transformed;
	float	color[3];
	float	intensity;
} dlight_t;

typedef struct
{
	float org[3];
	float vel[3];
	float accel[3];
	float time;
	unsigned color;
	float alpha;
} particle_t;

typedef struct
{
	float color[3];
} lightstyle_t;


typedef struct
{
	int			x, y, width, height;// in virtual screen coordinates
	float		fov_x, fov_y;
	float		vieworigin[3];
	float		viewangles[3];
	float		blend[4];			// rgba 0-1 full screen blend
	float		time;				// time is uesed to auto animate
	int			rdflags;			// RDF_UNDERWATER, etc

	byte		*areabits;			// if not NULL, only areas with set bits will be drawn

	lightstyle_t	*lightstyles;	// [MAX_LIGHTSTYLES]

	int			num_entities;
	entity_t	*entities;

	int			num_dlights;
	dlight_t	*dlights;

	int			num_particles;
	particle_t	*particles;
} refdef_t;


// refresh API imports
void Cmd_AddCommand (char *cmd_name, xcommand_t function);
int Cmd_Argc (void);
char *Cmd_Argv (int arg);
void Cbuf_ExecuteText (int exec_when, char *text);
void VID_Printf (int print_level, char *fmt, ...);
void VID_Error (int err_level, char *fmt, ...);
int FS_LoadFile (char *path, void **buffer);
void FS_FreeFile (void *buffer);
char *FS_Gamedir (void);
cvar_t *Cvar_Get (char *var_name, char *value, int flags);
cvar_t *Cvar_Set (char *var_name, char *value);
void Cvar_SetValue (char *var_name, float value);
void VID_MenuInit (void);
void VID_NewWindow (int width, int height);

void R_BeginRegistration (char *model);
struct model_s *R_RegisterModel (char *name);
struct image_s *R_RegisterSkin (char *name);
struct image_s *Draw_FindPic (char *name);
void R_SetSky (char *name, float rotate, vec3_t axis);
void R_EndRegistration (void);
void R_RenderFrame (refdef_t *fd);
void Draw_GetPicSize (int *w, int *h, char *pic);
void Draw_Pic (int x, int y, char *pic, float alpha = 1.0f);
void Draw_Pic (int x, int y, int w, int h, char *pic, float alpha = 1.0f);
void Draw_Char (int x, int y, int c);
void Draw_TileClear (int x, int y, int w, int h, char *name);
void Draw_Fill (int x, int y, int w, int h, int c);
void Draw_FadeScreen (void);
void Draw_StretchRaw (int x, int y, int w, int h, int cols, int rows, byte *data);
qboolean R_Init (void *hinstance, void *hWnd);
void R_Shutdown (void);
void R_SetPalette (const unsigned char *palette);
void R_BeginFrame (float camera_separation);
void GLimp_EndFrame (void);
void GLimp_AppActivate (qboolean active);
void VID_ResetDevice (void);
void D3D_InitModeList (void);
void Swap_Init (void);

#endif // __REF_H
